/*
  ==============================================================================

    This file was auto-generated!

    It contains the  code for a JUCE plugin processor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
ClassTestAudioProcessor::ClassTestAudioProcessor()
    :   parameters (*this, nullptr)
#ifndef JucePlugin_PreferredChannelConfigurations
     , AudioProcessor (BusesProperties()
                     #if ! JucePlugin_IsMidiEffect
                      #if ! JucePlugin_IsSynth
                       .withInput  ("Input",  AudioChannelSet::stereo(), true)
                      #endif
                       .withOutput ("Output", AudioChannelSet::stereo(), true)
                     #endif
                       )
#endif
{
    parameters.createAndAddParameter(std::make_unique<AudioParameterFloat>("gain",       // parameter ID
                                                                            "gain",       // parameter name
                                                                            NormalisableRange<float> (-60.0f, 12.0f),    // range
                                                                            0.0f,         // default value
                                                                            "dB"));
     /*
    AudioParameterFloat (const String& parameterID,
                         const String& name,
                         NormalisableRange<float> normalisableRange,
                         float defaultValue,
                         const String& label = String(),
                         Category category = AudioProcessorParameter::genericParameter,
                         std::function<String (float value, int maximumStringLength)> stringFromValue = nullptr,
                         std::function<float (const String& text)> valueFromString = nullptr);*/

    
    parameters.state = ValueTree (Identifier ("ClassTestVT"));

    gain = 1.0;
}

ClassTestAudioProcessor::~ClassTestAudioProcessor()
{
}

//==============================================================================
const String ClassTestAudioProcessor::getName() const
{
    return JucePlugin_Name;
}

bool ClassTestAudioProcessor::acceptsMidi() const
{
   #if JucePlugin_WantsMidiInput
    return true;
   #else
    return false;
   #endif
}

bool ClassTestAudioProcessor::producesMidi() const
{
   #if JucePlugin_ProducesMidiOutput
    return true;
   #else
    return false;
   #endif
}

bool ClassTestAudioProcessor::isMidiEffect() const
{
   #if JucePlugin_IsMidiEffect
    return true;
   #else
    return false;
   #endif
}

double ClassTestAudioProcessor::getTailLengthSeconds() const
{
    return 0.0;
}

int ClassTestAudioProcessor::getNumPrograms()
{
    return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
                // so this should be at least 1, even if you're not really implementing programs.
}

int ClassTestAudioProcessor::getCurrentProgram()
{
    return 0;
}

void ClassTestAudioProcessor::setCurrentProgram (int index)
{
}

const String ClassTestAudioProcessor::getProgramName (int index)
{
    return {};
}

void ClassTestAudioProcessor::changeProgramName (int index, const String& newName)
{
}

//==============================================================================
void ClassTestAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
{
    // Use this method as the place to do any pre-playback
    // initialisation that you need..
    phase = 0.0;
    twopi = 8.0 * atan(1.0);
}

void ClassTestAudioProcessor::releaseResources()
{
    // When playback stops, you can use this as an opportunity to free up any
    // spare memory, etc.
}

#ifndef JucePlugin_PreferredChannelConfigurations
bool ClassTestAudioProcessor::isBusesLayoutSupported (const BusesLayout& layouts) const
{
  #if JucePlugin_IsMidiEffect
    ignoreUnused (layouts);
    return true;
  #else
    // This is the place where you check if the layout is supported.
    // In this template code we only support mono or stereo.
    if (layouts.getMainOutputChannelSet() != AudioChannelSet::mono()
     && layouts.getMainOutputChannelSet() != AudioChannelSet::stereo())
        return false;

    // This checks if the input layout matches the output layout
   #if ! JucePlugin_IsSynth
    if (layouts.getMainOutputChannelSet() != layouts.getMainInputChannelSet())
        return false;
   #endif

    return true;
  #endif
}
#endif

void ClassTestAudioProcessor::processBlock (AudioBuffer<float>& buffer, MidiBuffer& midiMessages)
{
    int sample;
    ScopedNoDenormals noDenormals;
    auto totalNumInputChannels  = getTotalNumInputChannels();
    auto totalNumOutputChannels = getTotalNumOutputChannels();

    // In case we have more outputs than inputs, this code clears any output
    // channels that didn't contain input data, (because these aren't
    // guaranteed to be empty - they may contain garbage).
    // This is here to avoid people getting screaming feedback
    // when they first compile a plugin, but obviously you don't need to keep
    // this code if your algorithm always overwrites all the output channels.
    for (auto i = totalNumInputChannels; i < totalNumOutputChannels; ++i)
        buffer.clear (i, 0, buffer.getNumSamples());

    // This is the place where you'd normally do the guts of your plugin's
    // audio processing...
    // Make sure to reset the state if your inner loop is processing
    // the samples and the outer loop is handling the channels.
    // Alternatively, you can process the samples with the channels
    // interleaved by keeping the same state.
    gain = *parameters.getRawParameterValue("gain");
    gain = pow(10.0, gain*0.05);

    for (int channel = 0; channel < totalNumInputChannels; ++channel)
    {
        auto* outData = buffer.getWritePointer (channel);
        auto* inData = buffer.getReadPointer (channel);
        for(sample = 0; sample < buffer.getNumSamples(); sample++)
        {
            outData[sample] = inData[sample] * gain;
        }
            
        // ..do something to the data...
    }
}

//==============================================================================
bool ClassTestAudioProcessor::hasEditor() const
{
    return true; // (change this to false if you choose to not supply an editor)
}

AudioProcessorEditor* ClassTestAudioProcessor::createEditor()
{
    return new ClassTestAudioProcessorEditor (*this);
}

//==============================================================================
void ClassTestAudioProcessor::getStateInformation (MemoryBlock& destData)
{
    // You should use this method to store your parameters in the memory block.
    // You could do that either as raw data, or use the XML or ValueTree classes
    // as intermediaries to make it easy to save and load complex data.
}

void ClassTestAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
{
    // You should use this method to restore your parameters from this memory block,
    // whose contents will have been created by the getStateInformation() call.
}

//==============================================================================
// This creates new instances of the plugin..
AudioProcessor* JUCE_CALLTYPE createPluginFilter()
{
    return new ClassTestAudioProcessor();
}
